@@ -113,12 +113,14 @@ def convert_alleles(ts):
             ts.has_reference_sequence()
             and len(ts.reference_sequence.data) > 0
     )
+    if not has_refseq:
+        raise ValueError("Tree sequence must have a valid reference sequence.")
     # unfortunately, nucleotide mutations may be stacked (e.g., substitutions
     # will appear this way) and they don't appear in any particular order;
     # so we must guess which is the most recent, by choosing the one that
     # has the largest SLiM time, doesn't appear in the parent list, or has
     # the lagest SLiM ID.
-    slim_muts = np.repeat(0, ts.num_mutations)
+    nuc_inds = tables.mutations.metadata_vector(['mutation_list', 0, 'nucleotide'], dtype='int')
     num_stacked = np.array([len(m.metadata['mutation_list']) for m in ts.mutations()])
     for k in np.where(num_stacked > 1)[0]:
         mut = ts.mutation(k)
@@ -138,21 +140,15 @@ def convert_alleles(ts):
             )
         ]
         x.sort()
-        slim_muts[k] = x[-1][3]
-    # gah that was terrible, ok back to it
-    nuc_inds = np.array(
-            [m.metadata['mutation_list'][0]['nucleotide']
-                for k, m in zip(slim_muts, ts.mutations())],
-            dtype='int',
-    )
+        j = x[-1][3]
+        nuc_inds[k] = mut.metadata['mutation_list'][j]['nucleotide']
     if np.any(nuc_inds == -1):
         raise ValueError("All mutations must be nucleotide mutations.")
-    if not has_refseq:
-        raise ValueError("Tree sequence must have a valid reference sequence.")
-    aa = [ts.reference_sequence.data[k] for k in tables.sites.position.astype('int')]
     da = np.array(NUCLEOTIDES)[nuc_inds]
-    tables.sites.packset_ancestral_state(aa)
     tables.mutations.packset_derived_state(da)
+    k = tables.sites.position.astype('int')
+    aa = np.frombuffer(ts.reference_sequence.data.encode('utf-8'), dtype='S1')[k]
+    tables.sites.packset_ancestral_state(aa.tobytes().decode('utf-8'))
 
     return tables.tree_sequence()
 
@@ -208,8 +204,13 @@ def generate_nucleotides(ts, reference_sequence=None, keep=True, seed=None):
     tables.mutations.clear()
     sets = [[k for k in range(4) if k != i] for i in range(4)]
     states = np.full((ts.num_mutations,), -1)
+    k = tables.sites.position.astype('int')
+    aa_list = np.searchsorted(
+            NUCLEOTIDES,
+            np.frombuffer(tables.reference_sequence.data.encode('utf-8'), dtype='S1')[k],
+    )
     for site in ts.sites():
-        aa = NUCLEOTIDES.index(tables.reference_sequence.data[int(site.position)])
+        aa = aa_list[site.id]
         muts = {}
         for mut in site.mutations:
             if mut.parent == tskit.NULL:
@@ -608,6 +609,32 @@ def annotate_tables(tables, model_type, tick, cycle=None, stage="early", referen
     if reference_sequence is not None:
         tables.reference_sequence.data = reference_sequence
 
+def next_slim_mutation_id(ts):
+    '''
+    Returns the next SLiM mutation ID for this tree sequence. This is useful 
+    because if you want to add more mutations to your SLiM tree sequence using 
+    :func:`msprime.sim_mutations`, you may need to specify the parameter 
+    `next_id` in your :class:`msprime.SLiMMutationModel` to be larger than any 
+    existing mutation IDs. Setting `next_id` equal to the output of this 
+    function will allow the mutated tree sequence to be read in by SLiM.
+    To do this, recall that the "derived state" of SLiM's mutations are
+    comma-separated strings of mutation IDs; this function just parses all derived
+    states and returns one larger than the largest integer found. It will return an error
+    if it encounters derived states that are not comma-separated strings of integers.
+    '''
+    max_id = 0
+    for mut in ts.mutations():
+        ds = mut.derived_state
+        if len(ds) > 0:
+            for d in ds.split(","):
+                try:
+                    max_id = max(max_id, int(d))
+                except ValueError:
+                    raise ValueError(f"The derived state of a mutation ({ds}) in the tree "
+                                     "sequence is not a comma-separated list of values "
+                                     "coercible to int. This is not a valid SLiM tree sequence.")
+    return max_id + 1
+
 
 def _annotate_nodes_individuals(tables, age):
     '''
