@@ -1,7 +1,6 @@
 """
 Test cases for tree sequences.
 """
-import pickle
 import random
 import numpy as np
 import os
@@ -32,6 +31,57 @@ class TestSlimTime(tests.PyslimTestCase):
                 mut_time = max([x['slim_time'] for x in mut.metadata['mutation_list']])
                 assert mut_time == pyslim.slim_time(ts, mut.time, stage="early")
 
+class TestNextMutationID(tests.PyslimTestCase):
+    '''
+    Tests for the function that returns the largest SLiM mutation ID.
+    '''
+    def test_next_id(self, recipe):
+        ts = recipe["ts"]
+        mt_ids_str = ','.join(tskit.unpack_strings(ts.tables.mutations.derived_state, ts.tables.mutations.derived_state_offset))
+        mt_ids = [int(i or 0) for i in mt_ids_str.split(',')]
+        max_mt_id = max(mt_ids)
+        assert max_mt_id + 1 == pyslim.next_slim_mutation_id(ts)
+
+    @pytest.mark.parametrize('recipe', recipe_eq("adds_mutations"), indirect=True)
+    def test_reload_slim(self, recipe, helper_functions, tmp_path):
+        ts = recipe["ts"]
+        rts = self.do_recapitate(
+                    ts,
+                    recombination_rate=1e-8,
+                    ancestral_Ne=100,
+                    random_seed=875,
+        )
+        next_id = pyslim.next_slim_mutation_id(rts)
+        mts = msprime.sim_mutations(
+                rts,
+                rate=1e-4,
+                keep=True,
+                model=msprime.SLiMMutationModel(type=1, next_id=next_id),
+        )
+        assert mts.num_mutations > rts.num_mutations
+        rrts = helper_functions.run_slim_restart(
+                mts,
+                "restart_nucleotides_WF.slim",
+                tmp_path,
+                WF=False,
+        )
+        # nothing should change
+        assert pyslim.next_slim_mutation_id(mts) == pyslim.next_slim_mutation_id(rrts)
+        assert rrts.num_mutations == mts.num_mutations
+
+        def test_invalid_derived_state(self):
+            ts = msprime.sim_ancestry(
+                    4,
+                    sequence_length=10,
+                    population_size=10,
+                    random_seed=10,
+            )
+            mts = msprime.sim_mutations(ts,
+                    model="jc69",
+                    rate=0.5,
+                    random_seed=23)
+            with pytest.raises(ValueError, match="need to be coercible to int"):
+                pyslim.next_slim_mutation_id(mts)
 
 class TestRecapitate(tests.PyslimTestCase):
     '''
@@ -704,6 +754,25 @@ class TestConvertNucleotides(tests.PyslimTestCase):
         ntc.provenances.clear()
         assert tc == ntc
 
+    def scramble_mutations(self, ts):
+        # scramble order of mutations so that the most recent is not always first,
+        # since we don't have a reliable way to get that out of SLiM
+        rng = np.random.default_rng(123)
+        t = ts.dump_tables()
+        t.mutations.clear()
+        for m in ts.mutations():
+            a = np.array(m.derived_state.split(","))
+            ii = rng.permutation(len(a))
+            ml = [m.metadata['mutation_list'][i] for i in ii]
+            t.mutations.append(
+                    m.replace(
+                        derived_state=",".join(a[ii]),
+                        metadata={'mutation_list': ml}
+                    )
+            )
+        t.compute_mutation_parents()
+        return t.tree_sequence()
+
     def test_convert_alleles_errors(self):
         ts = msprime.sim_ancestry(4, sequence_length=10, population_size=10)
         with pytest.raises(ValueError, match="must have a valid reference sequence"):
@@ -741,6 +810,14 @@ class TestConvertNucleotides(tests.PyslimTestCase):
         cts = pyslim.convert_alleles(ts)
         self.verify_converted_nucleotides(ts, cts)
 
+    @pytest.mark.parametrize(
+            'recipe', recipe_eq("nucleotides", exclude="non-nucleotides"), indirect=True
+    )
+    def test_convert_alleles_scrambled(self, recipe):
+        ts = self.scramble_mutations(recipe["ts"])
+        cts = pyslim.convert_alleles(ts)
+        self.verify_converted_nucleotides(ts, cts)
+
     @pytest.mark.parametrize(
             'recipe', recipe_eq("nucleotides", exclude="non-nucleotides"), indirect=True
     )
